@@ -316,8 +316,8 @@ def one_of(*args):
 @defines_strategy_with_reusable_values
 def integers(min_value=None, max_value=None):
     # type: (Real, Real) -> SearchStrategy[int]
-    """Returns a strategy which generates integers (in Python 2 these may be
-    ints or longs).
+    """Returns a strategy which generates integers; in Python 2 these may be
+    ints or longs.
 
     If min_value is not None then all values will be >= min_value. If
     max_value is not None then all values will be <= max_value
@@ -333,6 +333,19 @@ def integers(min_value=None, max_value=None):
     min_int_value = None if min_value is None else ceil(min_value)
     max_int_value = None if max_value is None else floor(max_value)
 
+    if min_value != min_int_value:
+        note_deprecation(
+            'min_value=%r cannot be exactly represented as an integer, which '
+            'will be an error in a future version.'
+            % (min_value)
+        )
+    if max_value != max_int_value:
+        note_deprecation(
+            'max_value=%r cannot be exactly represented as an integer, which '
+            'will be an error in a future version.'
+            % (max_value)
+        )
+
     if min_int_value is not None and max_int_value is not None and \
             min_int_value > max_int_value:
         raise InvalidArgument('No integers between min_value=%r and '
@@ -440,6 +453,21 @@ def floats(
         max_value = float_of(max_value, width)
         assert isinstance(max_value, float)
 
+    if min_value != min_arg:
+        note_deprecation(
+            'min_value=%r cannot be exactly represented as a float of width '
+            '%d, which will be an error in a future version. Use min_value=%r '
+            'instead.'
+            % (min_value, width, min_arg)
+        )
+    if max_value != max_arg:
+        note_deprecation(
+            'max_value=%r cannot be exactly represented as a float of width '
+            '%d, which will be an error in a future version. Use max_value=%r '
+            'instead'
+            % (max_value, width, max_arg)
+        )
+
     check_valid_interval(min_value, max_value, 'min_value', 'max_value')
     if min_value == float(u'-inf'):
         min_value = None
@@ -602,7 +630,7 @@ def sampled_from(elements):
 def lists(
     elements=None,  # type: SearchStrategy[Ex]
     min_size=0,  # type: int
-    average_size=None,  # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
     unique_by=None,  # type: Callable[..., Any]
     unique=False,  # type: bool
@@ -672,7 +700,7 @@ def lists(
 def sets(
     elements=None,  # type: SearchStrategy[Ex]
     min_size=0,   # type: int
-    average_size=None,  # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
 ):
     # type: (...) -> SearchStrategy[Set[Ex]]
@@ -702,7 +730,7 @@ def sets(
 def frozensets(
     elements=None,  # type: SearchStrategy[Ex]
     min_size=0,   # type: int
-    average_size=None,  # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
 ):
     # type: (...) -> SearchStrategy[FrozenSet[Ex]]
@@ -789,7 +817,7 @@ def dictionaries(
     values,  # type: SearchStrategy[T]
     dict_class=dict,  # type: type
     min_size=0,  # type: int
-    average_size=None,  # type: int
+    average_size=None,  # type: None
     max_size=None,  # type: int
 ):
     # type: (...) -> SearchStrategy[Dict[Ex, T]]
@@ -865,7 +893,7 @@ def characters(
     - If ``whitelist_characters`` is specified, then any additional characters
       in that list will also be produced.
     - If ``blacklist_characters`` is specified, then any characters in
-      that list will be not be produced. Any overlap between \
+      that list will be not be produced. Any overlap between
       ``whitelist_characters`` and ``blacklist_characters`` will raise an
       exception.
 
@@ -896,8 +924,11 @@ def characters(
             blacklist_categories is None,
             )):
         raise InvalidArgument(
-            'Passing only whitelist_characters=%r would have no effect. '
-            'Perhaps you want sampled_from() ?' % (whitelist_characters,))
+            'Nothing is excluded by other arguments, so passing only '
+            'whitelist_characters=%(chars)r would have no effect.  Also pass '
+            'whitelist_categories=(), or use sampled_from(%(chars)r) instead.'
+            % dict(chars=whitelist_characters)
+        )
     blacklist_characters = blacklist_characters or ''
     whitelist_characters = whitelist_characters or ''
     overlap = set(blacklist_characters).intersection(whitelist_characters)
@@ -935,38 +966,40 @@ def characters(
 @cacheable
 @defines_strategy_with_reusable_values
 def text(
-    alphabet=None,  # type: Union[Sequence[Text], SearchStrategy[Text]]
+    alphabet=characters(
+        blacklist_categories=('Cs',)
+    ),  # type: Union[Sequence[Text], SearchStrategy[Text]]
     min_size=0,   # type: int
-    average_size=None,   # type: int
+    average_size=None,   # type: None
     max_size=None  # type: int
 ):
     # type: (...) -> SearchStrategy[Text]
     """Generates values of a unicode text type (unicode on python 2, str on
     python 3) with values drawn from alphabet, which should be an iterable of
-    length one strings or a strategy generating such. If it is None it will
-    default to generating the full unicode range (excluding surrogate
-    characters). If it is an empty collection this will only generate empty
-    strings.
+    length one strings or a strategy generating such strings.
+
+    The default alphabet strategy can generate the full unicode range but
+    excludes surrogate characters because they are invalid in the UTF-8
+    encoding.  You can use :func:`~hypothesis.strategies.characters` without
+    arguments to find surrogate-related bugs such as :bpo:`34454`.
 
     min_size and max_size have the usual interpretations.
+    Note that Python measures string length by counting codepoints: U+00C5
+    ``Å`` is a single character, while U+0041 U+030A ``Å`` is two - the ``A``,
+    and a combining ring above.
 
     The average_size argument is deprecated.  Internal upgrades since
     Hypothesis 1.x mean we no longer needed this hint to generate useful data.
 
     Examples from this strategy shrink towards shorter strings, and with the
     characters in the text shrinking as per the alphabet strategy.
+    This strategy does not :func:`~python:unicodedata.normalize` examples,
+    so generated strings may be in any or none of the 'normal forms'.
     """
     check_valid_sizes(min_size, average_size, max_size)
     if alphabet is None:
+        note_deprecation('alphabet=None is deprecated; just omit the argument')
         char_strategy = characters(blacklist_categories=('Cs',))
-    elif not alphabet:
-        if (min_size or 0) > 0:
-            raise InvalidArgument(
-                'Invalid min_size %r > 0 for empty alphabet' % (
-                    min_size,
-                )
-            )
-        return just(u'')
     elif isinstance(alphabet, SearchStrategy):
         char_strategy = alphabet
     else:
@@ -995,6 +1028,8 @@ def text(
                 'will be an error in future:  %r' % (not_one_char,)
             )
         char_strategy = sampled_from(alphabet)
+    if (max_size == 0 or char_strategy.is_empty) and not min_size:
+        return just(u'')
     return StringStrategy(lists(
         char_strategy, min_size=min_size, max_size=max_size
     ))
@@ -1004,7 +1039,7 @@ def text(
 @defines_strategy
 def from_regex(regex, fullmatch=False):
     # type: (Union[AnyStr, Pattern[AnyStr]], bool) -> SearchStrategy[AnyStr]
-    """Generates strings that contain a match for the given regex (i.e. ones
+    r"""Generates strings that contain a match for the given regex (i.e. ones
     for which :func:`python:re.search` will return a non-None result).
 
     ``regex`` may be a pattern or :func:`compiled regex <python:re.compile>`.
@@ -1022,11 +1057,11 @@ def from_regex(regex, fullmatch=False):
     mainly includes (positive or negative) lookahead and lookbehind groups.
 
     If you want the generated string to match the whole regex you should use
-    boundary markers. So e.g. ``r"\\A.\\Z"`` will return a single character
-    string, while ``"."`` will return any string, and ``r"\\A.$"`` will return
-    a single character optionally followed by a ``"\\n"``.
+    boundary markers. So e.g. ``r"\A.\Z"`` will return a single character
+    string, while ``"."`` will return any string, and ``r"\A.$"`` will return
+    a single character optionally followed by a ``"\n"``.
     Alternatively, passing ``fullmatch=True`` will ensure that the whole
-    string is a match, as if you had used the ``\\A`` and ``\\Z`` markers.
+    string is a match, as if you had used the ``\A`` and ``\Z`` markers.
 
     Examples from this strategy shrink towards shorter strings and lower
     character values, with exact behaviour that may depend on the pattern.
@@ -1163,7 +1198,7 @@ def builds(
         # Avoid an implementation nightmare juggling tuples and worse things
         raise InvalidArgument('infer was passed as a positional argument to '
                               'builds(), but is only allowed as a keyword arg')
-    required = required_args(target, args, kwargs)
+    required = required_args(target, args, kwargs) or set()
     to_infer = set(k for k, v in kwargs.items() if v is infer)
     if required or to_infer:
         if isclass(target) and attr.has(target):
@@ -1304,8 +1339,7 @@ def from_type(thing):
         return sampled_from(thing)
     # If we know that builds(thing) will fail, give a better error message
     required = required_args(thing)
-    if not any([
-        not required,
+    if required and not any([
         required.issubset(get_type_hints(thing.__init__)),
         attr.has(thing),
         # NamedTuples are weird enough that we need a specific check for them.
@@ -1362,6 +1396,7 @@ def fractions(
             assert max_denominator is not None
             if value is None or value.denominator <= max_denominator:
                 return value, value
+
             p0, q0, p1, q1 = 0, 1, 1, 0
             n, d = value.numerator, value.denominator
             while True:
@@ -1379,8 +1414,22 @@ def fractions(
         # Take the high approximation for min_value and low for max_value
         bounds = (max_denominator, min_value, max_value)
         if min_value is not None:
+            if min_value.denominator > max_denominator:
+                note_deprecation(
+                    'The min_value=%r has a denominator greater than the '
+                    'max_denominator=%r, which will be an error in a future '
+                    'version.'
+                    % (min_value, max_denominator)
+                )
             _, min_value = fraction_bounds(min_value)
         if max_value is not None:
+            if max_value.denominator > max_denominator:
+                note_deprecation(
+                    'The max_value=%r has a denominator greater than the '
+                    'max_denominator=%r, which will be an error in a future '
+                    'version.'
+                    % (max_value, max_denominator)
+                )
             max_value, _ = fraction_bounds(max_value)
 
         if min_value is not None and max_value is not None and \
@@ -1650,7 +1699,7 @@ def datetimes(
     # that took aware bounds, but the API and validation is much harder.
     # If you want to generate datetimes between two particular momements in
     # time I suggest (a) just filtering out-of-bounds values; (b) if bounds
-    # are very close, draw a value and subtract it's UTC offset, handling
+    # are very close, draw a value and subtract its UTC offset, handling
     # overflows and nonexistent times; or (c) do something customised to
     # handle datetimes in e.g. a four-microsecond span which is not
     # representable in UTC.  Handling (d), all of the above, leads to a much
